﻿using System;
using Microsoft.Xna.Framework;

namespace Petzold.Astronomy
{
    public struct EclipticCoordinate
    {
        public EclipticCoordinate(Angle longitude, Angle latitude, double radius) : this()
        {
            this.Longitude = longitude;
            this.Latitude = latitude;
            this.Radius = radius;
        }

        public EclipticCoordinate(Vector3 point) : this()
        {
            this.RectangularCoordinates = point;
        }

        public Angle Longitude { private set; get; }

        public Angle Latitude { private set; get; }

        public double Radius { private set; get; }

        public Vector3 RectangularCoordinates
        {
            set
            {
                this.Radius = Math.Sqrt(value.X * value.X +
                                        value.Y * value.Y +
                                        value.Z * value.Z);

                this.Longitude = Angle.ArcTangent(-value.Z, value.X);
                this.Latitude = Angle.ArcTangent(value.Y, Math.Sqrt(value.X * value.X + value.Z * value.Z));
            }
            get
            {
                double x = Radius * Latitude.Cosine * Longitude.Cosine;
                double y = Radius * Latitude.Sine;
                double z = -Radius * Latitude.Cosine * Longitude.Sine;

                return new Vector3((float)x, (float)y, (float)z);
            }
        }

        public override string ToString()
        {
            return String.Format("({0}, {1}, {2})", this.Longitude, this.Latitude, this.Radius);
        }
    }
}
